/* celloutG.c -- cellular automata/motif interface for RTcmix */
#define MAIN
#include <globals.h>
#include "RTcmix.h"

#include <Xm/Xm.h>
#include <Xm/BulletinB.h>
#include <Xm/DrawingA.h>

#include <stdio.h>
#include <unistd.h>

Widget topLevel, bboard;
Widget theDraw;

XtAppContext app;

GC gc;
Colormap cmap;
XColor bground,fground,unused;

#define NCELLS 16
#define NGENS 40
#define CWIDTH 20

int drawwidth,drawheight;

/* music stuff */
int cells[2][NCELLS];
int cur,next;
float beat;
int currentgen;
float pitches[10] = { 6.10, 7.05, 7.07, 7.10, 8.00, 8.02, 8.03, 8.05, 8.07, 8.10 };


RTcmix *rrr; // the RTcmix object


int
main(int argc, char *argv[])
{
	void *gocells();
	float brrand();
	void srrand();
	float fval;
	int i;

	rrr = new RTcmix(44100.0, 2);
	sleep(1);
	rrr->cmd("load", 1, "STRUM");

/* and now the graphics stuff */
	topLevel=XtVaAppInitialize(&app,"CellautoG",
		NULL,0,&argc,argv,NULL,NULL);

	/* set up the main window */
	drawwidth = NCELLS * CWIDTH;
	drawheight = NGENS * CWIDTH;
	bboard = XtVaCreateManagedWidget("bboard",
		xmBulletinBoardWidgetClass, topLevel,
		XmNwidth, drawwidth, XmNheight, drawheight,
		XmNmarginWidth, 0, XmNmarginHeight, 0,NULL);

	/* set up the colors */
	cmap = DefaultColormapOfScreen(XtScreen(bboard));
	XAllocNamedColor(XtDisplay(bboard),cmap,"deepskyblue",&bground,&unused);
	XAllocNamedColor(XtDisplay(bboard),cmap,"deeppink3",&fground,&unused);
	gc = XCreateGC(XtDisplay(bboard),
		RootWindowOfScreen(XtScreen(bboard)), 0, NULL);

	/* set up the drawing area in the main window */
	theDraw = XtVaCreateManagedWidget("theDraw",
		xmDrawingAreaWidgetClass, bboard,
		XmNwidth, drawwidth, XmNheight, drawheight,
		XmNbackground, bground.pixel, NULL);

	/* set up the music params */
	beat = 0.1;
	currentgen = 0;

	/* initialize the first row */
	cur = 0;
	next = 1;
	for (i = 0; i < NCELLS; i++) {
		fval = brrand();
		if (fval > 0.5) cells[cur][i] = 1;
		else cells[cur][i] = 0;
	}

/* set up the timer */
	RTtimeit(beat*(float)NCELLS, (sig_t)gocells);

	XtManageChild(bboard);
	XtManageChild(theDraw);
	XtRealizeWidget(topLevel);

	XtAppMainLoop(app);
}



void gocells()
{
	int j;
	float brrand();
	int right,left,combine;

	/* go through the cells */
	XSetForeground(XtDisplay(theDraw), gc, fground.pixel);
	for (j = 0; j < NCELLS; j++) {

		if (cells[cur][j] == 1) {
			/* draw it! */
			XFillRectangle(XtDisplay(theDraw),XtWindow(theDraw),gc,
				j*CWIDTH,currentgen*CWIDTH,CWIDTH,CWIDTH);

			/* sound it! */
			rrr->cmd("START", 7,
				(float)j*beat, 0.5, 9.00, 0.5, 0.3, 7000.0, 0.0);

/*
pitches[(int)(brrand() * 10.0)]
*/
		}
		else {
			/* sound it! */
			rrr->cmd("START", 7,
				(float)j*beat, 0.5, 7.00, 0.5, 0.3, 5000.0, 3.0);
		}

		/* now calculate the next row element */
		/* find the neighbors */
		if (j == 0) left = 0;
		else left = cells[cur][j-1];
		if (j == NCELLS-1) right = 1;
		else right = cells[cur][j+1];

		combine = left + right;
		if (combine == 0) {
			cells[next][j] = 0;
		}
		if (combine == 1) {
			cells[next][j] = 1;
		}
		if (combine == 2) {
			cells[next][j] = 0;
		}
	}
	/* now set for the next round */
	cur = next;
	if (++next > 1) next = 0;
	if (++currentgen > NGENS) {
		currentgen = 0;
		XSetForeground(XtDisplay(theDraw), gc, bground.pixel);
		XFillRectangle(XtDisplay(theDraw),XtWindow(theDraw),gc,
			0,0,NCELLS*CWIDTH,NGENS*CWIDTH);
	}

	XFlush(XtDisplay(theDraw));
}



long randx = 1;

void srrand(int x)
{
	randx = x;
}

float brrand()
{
	int i = ((randx = randx*1103515245 + 12345)>>16) & 077777;
	return((float)i/32768.0);
}



